#include <malloc.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

double *p=NULL;
int init=1;

void lddif (double b[],double z[],double y[],double e[],int m,int n, double C,double L,double d, double *initp)
  /*
    Identification LDDIF by Kulhavy and Karny, Budapest 1984

    b[n*m]           parameter vector
    z[n]             data vector
    y[m]             output vector
    e[m]             prediction error vector
    m                number of outputs
    n                number of ident. data

    date   :         03.12.1991
    author :         M.Fikar
  */
{
  
  double l=L;                    /* l is exponential forgetting factor */
  double delta = d;              /* delta is lower bound on tr(p) */
  double s0,s1,s2,s3,s4,s5;
  register int i,j;
  if (init)
  {
    init--;
    p= (double *) calloc( (size_t)((n+2)*n), (size_t)sizeof(double) );
    if ( p == NULL)
    {
      printf("function lddif : ERROR in memory allocation \n");
      exit(0);
    }
      
    for (i=0 ; i<n ; i++)
      p[i*n+i] = C;
      
    for (i=0 ; i<m*n ; i++)
        b[i]=initp[i];
      //b[i]=.1*(i+1);
      
  }

  p[n*n+n-1] = p[(n-1)*n+n-1] * z[n-1]; p[(n + 1)*n+n-1] = p[n*n+n-1] * z[n-1];
  s2 = p[(n + 1)*n+n-1];
  for (i=0 ; i<m ; i++)
    e[i] = y[i] - z[n-1] * b[(i+1)* n-1];       
  for (j=n-2 ; j>=0 ; j--)
  {
    for (i=0 ; i<m ; i++)
      e[i] -= z[j] * b[j+n*i];
    s0 = z[j];
    for (i=j+1 ; i<n ; i++)
      s0 += p[i*n+j] * z[i];
    p[n*n+j] = s0 * p[j*n+j];
    s2 += s0 * p[n*n+j]; p[(n + 1)*n+j] = s2;
  }
  if (s2 < .000001) return;
  s3 = l * s2 + l - 1;
  if (fabs(s3) < .0001)
  {
    for (j=n-1 ; j>=1 ; j--)
      for (i=j+1 ; i<=n ; i++)
        p[n*n+i-1] += p[(i-1)*n+j-1] * p[n*n+j-1];
  }
  else
  {
    s3 = s2 / s3; s1 = p[(n + 1)*n+n-1] + s3; p[(n-1)*n+n-1] *= s3 / s1+delta;
    for (j=n-2 ; j>=0 ; j--)
    {
      s5 = p[j*n+j] * s1; s4 = p[n*n+j] / s5;
      s1 = p[(n + 1)*n+j] + s3; p[j*n+j] = s5 / s1+delta;
      for (i=j+1 ; i<n ; i++)
      {
        s5 = p[i*n+j]; p[i*n+j] = s5 - s4 * p[n*n+i];
        p[n*n+i] += s5 * p[n*n+j];
      }
    }
  }
  for (i=0 ; i<m ; i++)
  {
    s0 = e[i] / (1 + s2);
    for (j=0 ; j<n ; j++)
      b[j + n * i] += p[n*n+j] * s0;
  }
}
